/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.auth;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.opengroup.osdu.core.common.entitlements.AuthorizationServiceImpl;
import org.opengroup.osdu.core.common.entitlements.IEntitlementsFactory;
import org.opengroup.osdu.core.common.entitlements.IEntitlementsService;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.entitlements.AuthorizationResponse;
import org.opengroup.osdu.core.common.model.entitlements.EntitlementsException;
import org.opengroup.osdu.core.common.model.entitlements.GroupInfo;
import org.opengroup.osdu.core.common.model.entitlements.Groups;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.powermock.modules.junit4.PowerMockRunner;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@RunWith(PowerMockRunner.class)
public class AuthorizationServiceEntitlementsTest {

    @Mock
    private IEntitlementsFactory entitlementsFactory;

    @Mock
    private IEntitlementsService service;

    @Mock
    private JaxRsDpsLog log;

    @InjectMocks
    private AuthorizationServiceImpl sut;

    @Before
    public void setup() {
        when(entitlementsFactory.create(any())).thenReturn(service);
    }

    @Test
    public void should_returnUser_when_userHasPermission() throws EntitlementsException {
        sut = createSut("service.register.user");
        AuthorizationResponse result = sut.authorizeAny(DpsHeaders.createFromMap(new HashMap<>()), "service.register.user");

        assertEquals("akelham@bbc.com", result.getUser());
    }

    @Test
    public void should_returnUser_when_userHasAnyPermission() throws EntitlementsException {
        sut = createSut("service.register.editor");
        AuthorizationResponse result = sut.authorizeAny(DpsHeaders.createFromMap(new HashMap<>()), "service.register.user", "service.register.editor");

        assertEquals("akelham@bbc.com", result.getUser());
    }

    @Test
    public void should_throwUnauthorized_when_userDoesNotHaveRequiredPermission() throws EntitlementsException {
        sut = createSut("service.register.user");
        try {
            sut.authorizeAny(DpsHeaders.createFromMap(new HashMap<>()), "service.register.editor");
            fail("expected exception");
        } catch (AppException ex) {
            assertEquals(401, ex.getError().getCode());
        }
    }

    @Test
    public void should_throwServerError_when_getGroupsThrowsServerError() throws EntitlementsException {
        sut = createSut("service.register.user");
        HttpResponse response = new HttpResponse();
        response.setResponseCode(500);
        when(service.getGroups()).thenThrow(new EntitlementsException("", response));
        try {
            sut.authorizeAny(DpsHeaders.createFromMap(new HashMap<>()), "service.register.editor");
            fail("expected exception");
        } catch (AppException ex) {
            assertEquals(500, ex.getError().getCode());
        }
    }

    @Test
    public void should_throw400AppError_when_getGroupsThrows400EntitlementsError() throws EntitlementsException {
        sut = createSut("service.register.user");
        HttpResponse response = new HttpResponse();
        response.setResponseCode(400);
        when(service.getGroups()).thenThrow(new EntitlementsException("", response));
        try {
            sut.authorizeAny(DpsHeaders.createFromMap(new HashMap<>()), "service.register.editor");
            fail("expected exception");
        } catch (AppException ex) {
            assertEquals(400, ex.getError().getCode());
        }
    }

    private AuthorizationServiceImpl createSut(String... roles) throws EntitlementsException {
        List<GroupInfo> groupInfos = new ArrayList<>();

        for (String s : roles) {
            GroupInfo group = new GroupInfo();
            group.setName(s);
            groupInfos.add(group);
        }
        Groups output = new Groups();
        output.setMemberEmail("akelham@bbc.com");
        output.setGroups(groupInfos);
        when(service.getGroups()).thenReturn(output);
        return sut;
    }

}

